from ae.maze_keeper import ACTIONS
import random


class Agent:
    """
    Agent intelligence for searching the maze.
    Receives size of the maze and the maximum number of steps to find the gold in the maze.
    Agent is supplied with its position in the maze, his field of vision, gold position and number of steps passed in
    the maze when the action method is called. Based on this information, agent has to decide which action to take.
    """

    def __init__(self, maze_size, step_limit, start_position, gold_position):
        self.step_limit = step_limit
        self.maze_size = maze_size
        self.gold_position = gold_position
        self.start_position = start_position
        self.state = None

    @staticmethod
    def random_action():
        """Moves randomly"""
        return random.choice(list(ACTIONS.keys()))

    @staticmethod
    def heuristic_random_action(state, gold_position):
        """Has better chance to move in the direction of the gold_position"""
        diff = tuple(v1 - v2 for v1, v2 in zip(gold_position, state.position))
        action_weights = {}
        if diff[1] > 0:
            action_weights['EAST'] = 0.4
            action_weights['WEST'] = 0.1
        else:
            action_weights['WEST'] = 0.4
            action_weights['EAST'] = 0.1

        if diff[0] > 0:
            action_weights['SOUTH'] = 0.4
            action_weights['NORTH'] = 0.1
        else:
            action_weights['NORTH'] = 0.4
            action_weights['SOUTH'] = 0.1

        actions, weights = zip(*action_weights.items())
        return random.choices(actions, weights)[0]

    def action(self, state):
        """
        Returns action the agent takes in given state
        :param state: State of the agent
        :return: One of the actions from ACTIONS.keys()

        ===YOUR AGENT DECISION MAKING CODE GOES HERE===
        """
        return self.heuristic_random_action(state, self.gold_position)
