from student_template.agent import Agent
from student_template.maze_generator import MazeGenerator
from ae.maze_keeper import MazeKeeper
from student_template import visualization


class Simulation:
    """
    Simulation class that controls the main simulation loop and visualization of the agents movement in the maze.
    Run this file from console for visualization to work properly.
    """
    def __init__(self, maze_size=(12, 10), step_limit=1000, visualize=True):
        self.visualize = visualize
        self.step_limit = step_limit
        self.maze_size = maze_size

        self.maze_generator = MazeGenerator(maze_size, step_limit)
        self.layout = self.maze_generator.generate()

        self.maze_keeper = MazeKeeper(self.layout)


        self.agent = Agent(maze_size, step_limit, self.maze_keeper.agent_position, self.maze_keeper.gold_position)

    def run(self):
        state = self.maze_keeper.state()
        step = 0
        trace = []
        while not self.maze_keeper.finished and step < self.step_limit:
            step += 1
            action = self.agent.action(state)
            state = self.maze_keeper.agent_move(action)
            trace.append(state.position)

        return trace, self.maze_keeper.layout

    def run_and_display_results(self):

        trace, layout = self.run()

        print('Simulation finished.')
        if self.maze_keeper.finished:
            print('Agent SUCCESSFULY found the gold', end='')
        else:
            print('Agent FAILED to find the gold', end='')
        print(' in {} steps.'.format(len(trace)))
        input('Press any key to visualize the agents progress.')
        if self.visualize:
            vis = visualization.Visualization(layout, speed=0.1)
            vis.show_trace(trace)


if __name__ == '__main__':
    sim = Simulation()
    sim.run_and_display_results()
